package com.loit.Proxy;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.BeanDefinitionRegistryPostProcessor;
import org.springframework.beans.factory.support.GenericBeanDefinition;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.ResourceLoaderAware;
import org.springframework.core.io.ResourceLoader;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 用于Spring动态注入自定义接口
 * 继承BeanDefinitionRegistryPostProcessor,ResourceLoaderAware,ApplicationContextAware，实现项目启动自动执行
 * 
 * @author Demon add by 2020-05-26
 */
@Component
public class ApiProxyRegistry implements BeanDefinitionRegistryPostProcessor,ResourceLoaderAware,ApplicationContextAware {
 
	/**
	 * 扫描包内所有接口，通过ApiProxyBeanFactory的动态代理产生接口的实现类并批量注册到spring中
	 */
    @Override
    public void postProcessBeanDefinitionRegistry(BeanDefinitionRegistry registry) throws BeansException {
    	
        // 通过反射获取需要代理的接口的clazz列表
    	List<Class<?>> beanClazzs = ApiProxyPackageScan.getClasses(ApiProxyConfig.PACKAGE_SCAN);
    	
    	// 将扫描到的接口列表循环注入到spring的管理中
        for (Class beanClazz : beanClazzs) {
            BeanDefinitionBuilder builder = BeanDefinitionBuilder.genericBeanDefinition(beanClazz);
            GenericBeanDefinition definition = (GenericBeanDefinition) builder.getRawBeanDefinition();
            
			/* 
			 * 在这里，我们可以给该对象的属性注入对应的实例。
			 * 比如mybatis，就在这里注入了dataSource和sqlSessionFactory，
			 * 注意，如果采用definition.getPropertyValues()方式的话，
			 * 类似definition.getPropertyValues().add("interfaceType", beanClazz);
			 * 则要求在FactoryBean（本应用中即ApiProxyBeanFactory）提供setter方法，否则会注入失败
			 * 如果采用definition.getConstructorArgumentValues()，
			 * 则FactoryBean中需要提供包含该属性的构造方法，否则会注入失败
			 */
            definition.getConstructorArgumentValues().addGenericArgumentValue(beanClazz);
            
            /*
             * 注意，这里的BeanClass是生成Bean实例的工厂，不是Bean本身。
             * ApiProxyBeanFactory是一种特殊的Bean，其返回的对象不是指定类的一个实例，
             * 其返回的是该工厂Bean的getObject方法所返回的对象。
             */
            definition.setBeanClass(ApiProxyBeanFactory.class);
 
            //这里采用的是byType方式注入，类似的还有byName等
            definition.setAutowireMode(GenericBeanDefinition.AUTOWIRE_BY_TYPE);
            registry.registerBeanDefinition(beanClazz.getSimpleName(), definition);
        }
    }

	@Override
	public void postProcessBeanFactory(ConfigurableListableBeanFactory arg0) throws BeansException {}

	@Override
	public void setApplicationContext(ApplicationContext arg0) throws BeansException {}

	@Override
	public void setResourceLoader(ResourceLoader arg0) {}

}
